<p align="center">
  <img src="https://github.com/Lartu/ldpl/blob/master/images/ldpl-4.0-logo.png">
  <br><br>
  <img src="https://img.shields.io/badge/license-apache_2.0-yellow?style=flat-square">
  <img src="https://img.shields.io/badge/current_release-4.3-blue.svg?style=flat-square">
  <img src="https://img.shields.io/badge/dev_version-4.4-purple.svg?style=flat-square">
  <a href='https://docs.ldpl-lang.org/'><img src="https://img.shields.io/badge/docs-up_to_date-green?style=flat-square"></a>
  <a href="https://www.freenode.net"><img src="https://img.shields.io/badge/irc-%23ldpl-orange?style=flat-square"></a>
  <a href="https://travis-ci.org/Lartu/ldpl"><img src="https://img.shields.io/travis/Lartu/ldpl/master?style=flat-square"></a>
</p>

[**LDPL**](https://www.ldpl-lang.org/) is a powerful transpiled programming language designed from the ground up to be excessively **expressive**, **readable**, **fast** and **easy** to learn. It mimics plain English, in the likeness of the good parts of older programming languages like COBOL, with the desire that it can be understood by anybody. It's very portable and runs on a plethora of different architectures and operating systems including AMD-64 Linux, macOS, ARMv8 Linux, Android Termux and both Intel and PowerPC OS X (*tested from Tiger 10.4 onwards*). It even supports UTF-8 out of the box.

This repository contains the source code and [releases](https://github.com/Lartu/ldpl/releases) of the LDPL compiler, LPM (the LDPL Package Manager) and other useful goodies.

### 📒 Example LDPL code

```coffeescript
# Leap Year Example
# Asks the user to enter a year and tells them if
# that's a leap year or not.
data: 
  year is number
  divBy4 is number
  divBy100 is number
  
procedure: 
  display "Enter a year: "
  accept year
  modulo year by 4 in divBy4
  modulo year by 100 in divBy100
  if divBy4 is not equal to 0 then
    display year " is a common year." lf
  else if divBy100 is not equal to 0 then
    display year " is a leap year!" lf
  else if divBy4 is not equal to 0 or divBy100 is not equal to 0 then
    display year " is a common year." lf
  else
    display year " is a leap year!" lf
  end if
```

This code asks the user to enter a year and then tells them if it is a leap year or not. Easy as pie and super legible. Check the [official website](https://www.ldpl-lang.org/) and the [examples](/examples) subfolder for more examples, including a Brainf*ck interpreter and Bellman-Ford's Algorithm!

### 🎏 What can I use LDPL for?

LDPL can be used to write software just like any other language you know. You can open, edit and save files, do math,
parse and modify text, etc. Some of these tasks are made easier by the fact that LDPL natively knows how to work with
UTF-8 strings. By using the [available libraries](https://github.com/Lartu/ldpl#-some-other-ldpl-libraries), LDPL becomes
a language *specially* well suited for writing **Telegram Bots**, **IRC Bots** and **Simple Network Servers**.
Code written in LDPL is easy to read, runs fast and can easily interface with C++ code,
so you can do anything you want, really.

## 📯 LDPL Philosophy

LDPL is a language designed to be easy to understand and simple to write. Designed to be easy to learn and simple to use.
We believe that coding should be easy and simple. Nothing should become a frustrating chore. Compiling source code should be easy and
simple: a single, flagless command should be enough to compile any source code. Every statement in the language should do 
one and one thing only, not multiple things depending on the context. Complex, low level stuff like encoding, sockets, 
floating point number comparison, etc., should be dealt by the compiler, transparently to the user. Hard stuff should be
made easier.

We understand that this philosphy may lead to longer source code, more verbose statements and extra steps to reach an end,
but we also believe that it should make coding in LDPL easier and more enjoyable than in other languages.

As one user once put it, *"Usually when I'm programming, I feel like I'm in a big fancy jet and there's a lot of turbulence and it's not going well but then all of a sudden it's smooth air again and the drink cart comes along and I get a ginger ale and it's great. But with LDPL, I feel like I'm a cub scout out in the woods with a box of matches and a hatchet and my Scout's Handbook (the LDPL Docs) just exploring and figuring it out as I go. Whenever I run into a problem I just check my handbook and, sure enough, there's a solution right there waiting for me!"*.

We want to make LDPL a language you'll love not because it lets you do many things in one line, nor because of the way
it runs, nor because of how modern it is: but because it's designed to stay by your side and tell you everything's gonna
be okay even when times get dark.

## 💾 How to install LDPL

### Using the LDPL Install Wizard:

You can use the **LDPL Install Wizard** to make and install LDPL. This is the **recommended install method**, as it sets up everything LDPL needs for you and installs the latest LDPL version available. This method requires *whiptail* to be installed on your system (it probably is).

<img src="https://github.com/Lartu/ldpl/blob/master/images/wizard.png" width=500px>

Just `git clone` this repository (or download it from [here](https://github.com/Lartu/ldpl/archive/master.zip) and un-zip it somewhere), open a terminal in it (or `cd` to it), run `sh install.sh` and follow the steps. Done!

### By Yourself:

You can get yourself a precompiled LDPL binary or compile LDPL from source. Check the following table if you are not sure of what to do.

| **If you want to download a compiled binary** | **If you want to build LDPL yourself** |
| ---                                           | --- |
| Compiled binaries contain the latest stable release. Might not be up to date with the newest, bleeding edge features, but will work fine on most systems and are already compiled for you. | Building LDPL yourself gives you access to the latest features of the language. Development LDPL code tends to be stable, compile and work fine, but hidden bugs may still lurk in there! |
| **How to:** [download the latest stable release available](https://github.com/Lartu/ldpl/releases). You should then move the binary to a folder on your PATH. | **How to:** first, clone this repository. Then `make` and `make install` LDPL in the `src` folder. This will install LDPL and the LDPL documentation (`man ldpl`) on your system. LDPL requires only C++11 to compile. |

Once you have LDPL installed on your system, check the LDPL reference to learn how to use the language. Information on how to compile LDPL scripts and a list of LDPL compatible editors is provided in the sections below.

## 📝 LDPL compatible editors

![VSC](https://www.ldpl-lang.org/images/vsc.png?v=1) ![VIM](https://www.ldpl-lang.org/images/vim.png) ![ATOM](https://www.ldpl-lang.org/images/atom.png) ![nano](https://www.ldpl-lang.org/images/nanologo.png)

We have highlighting and code formatting extensions available for [Visual Studio Code](https://marketplace.visualstudio.com/items?itemName=Lartu.ldpl-vsc-extension) (thanks to [dvkt](https://github.com/dvkt) and [Lartu](http://github.com/lartu)), [vim](https://github.com/araml/ldpl.vim) (thanks to [Araml](https://github.com/araml)), [Atom](https://atom.io/packages/language-ldpl) (thanks to [Damián Garro](https://github.com/dgarroDC)) and [nano](https://github.com/Lartu/ldpl/tree/master/highlighting/nano) (by [Lartu](http://github.com/lartu)).

You can use any other editor (without syntax highlighting) if you like. If know how to write syntax packages for other editors and would like to contribute to the LDPL project, a highlighting extension would be a very welcome contribution.

## 📚 Learning LDPL and Read the Docs

![Learning Dinosaur](https://github.com/Lartu/ldpl/blob/master/images/reference-logo.png)

If you want to learn how to code in LDPL, there's a little **tutorial** at https://learnxinyminutes.com/docs/ldpl/.

Also be sure to check the [examples](./examples)!

[The **LDPL documentation** is available here](https://docs.ldpl-lang.org/). Check that to learn yourself some LDPL (*for the greater good!*). The documentation is also uploaded to the [docs](docs) folder of this repository and can be read and modified from there.

The LDPL documentation can also be found on your system using `man ldpl` when you install LDPL by using `make install` or the LDPL Install Wizard. The man page is also available in the [man](/man) folder of this repository.

## 💻 How to use this compiler

To use the compiler, you must have a C++ compiler already installed on your system and have mapped it to `c++`, found on your `PATH`. The LDPL Compiler compiles LDPL code to C++ code and thus this is a requirement for it to work.

Once the compiler is set up, go write some LDPL source code, say `source.lsc`.
Then compile the source code using `ldpl source.lsc`. The compiled, executable binary file will be saved as `source-bin`.
Done! For more info on the compiler run `ldpl -h`.

For more information, check the [docs](https://docs.ldpl-lang.org/#the-ldpl-compiler).

### 🔧 Compiler options

 * The `-f` flag can be used to pass extra options to the compiler when building extensions. For example, `-f=-lSDL` could be used to link against SDL. The `flag` statement can also be used as well (`flag "-lSDL"`) and its use is recommended.
 * By using `-r` you can just compile the project and print the C++ representation for that code.
 * You can set the output file for the compiled binary with the `-o` flag. For example, if you want to name your program "dog", you could compile it with `ldpl -o=dog main.ldpl`.
 * On Linux platforms, LDPL builds static binaries by default. If you want to build non-static ones use the `-ns` flag.
 * The `-c` flag tells LDPL to accept source code from the standard input. 
 * You can import extensions to your LDPL compilation by using the `-i=` flag. Extensions can be imported by passing `.o`, `.a`, or `.cpp` files to this flag; see the Extensions section below.
 * `-v` and `--version` print out version info and release details.
 * `-h` and `--help` print this list of options.
 
 For more information, check the [docs](https://docs.ldpl-lang.org/#compiler-switches).

### 📦 C++ extensions

LDPL supports extensions written in C++. Extensions are `.cpp`, `.o`, or `.a` files that can be imported into your program using the `EXTENSION` statement. For example `extension "myLibrary.cpp"`. You can also use the `-i` compiler flag, but this is not recommended for ease of distribution.

For a guide to writing and building C++ extensions, see the [LDPL official documentation](https://docs.ldpl-lang.org/cppext/). For a simple example, see the [examples/cpp-extension](./examples/cpp-extension) directory.

## 🏛 Libraries

![LPM Logo](images/lpm-logo.png)

Starting from version 4.3, LDPL includes the **LDPL Package Manager** (LPM). When you install LDPL from source you also install LPM. If you download a pre-compiled binary, LPM is included in the same package. With LPM, downloading and using libraries (we also call them *packages*) is now easier than ever:
 - Run `lpm info <library_name>` to find information about a particular library.
   - Try, for example, `lpm info std-math` and `lpm info ltb`!
 - Run `lpm install <library_name>` to install a particular library.
 - Run `lpm uninstall <library_name>` to uninstall a particular library.
 - Run `lpm count` to display the number of installed packages.
 - Run `lpm list` to list all installed packages.
 
Once you've installed a library, you can include it to your project by just adding the line

```coffeescript
using package <library_name> # for example: using package std-math
```

before the `DATA` and `PROCEDURE` sections of your source file.

Libraries not installed using LPM can be included using

```coffeescript
include "library_name" # for example: include "std-math"
```

before the `DATA` and `PROCEDURE` sections of your source file.

### 📖 The LDPL Standard Library

A number of useful statements and subroutines are available in the [**LDPL Standard Library**](https://github.com/Lartu/ldpl-std). Just `lpm install` and include with `using package` the desired packages in your sourcecode and you are ready to go. For more information check the LDPL Standard Library repository.

### 🦕 Other LDPL Libraries

- [**The LDPL Network Server Library**](https://github.com/Lartu/ldpl-net-server/): the LDPL Network Server Library is an library for creating socket based servers in LDPL. It aims to make it very easy to develop, test and deploy network servers, with as little work as possible. Install with: `lpm install ldpl_net_server`.
- [**The LDPL Socket Library**](https://github.com/dvkt/ldpl-socket): the LDPL socket Library allows you to open, close, write to, and read from network sockets in LDPL. Install with: `lpm install ldpl_socket`.
- [**The LDPL IRC Bot Library**](https://github.com/Lartu/ldpl-irc-bot): the LDPL IRC Bot Library lets you write LDPL programs that connect to IRC servers, join channels, send messages and more in the simplest
way possible. Install with: `lpm install ldpl_irc_bot`.
- [**The LDPL Ncurses Library**](https://github.com/Lartu/ldpl-ncurses): the LDPL Ncurses Library is a ncurses wrapper for LDPL, designed for creating terminal applications that require more complex text user-interfaces than simple line-feeding. Install with: `lpm install ldpl-ncurses`.
- [**The LDPL Telegram Bot Library**](https://github.com/dgarroDC/ltb): this simple LDPL library lets you create a Telegram bot that can receive and send text messages. Install with: `lpm install ltb`.

## 👨🏼‍💻 How can I contribute to LDPL?

If you want to contribute to LDPL you can fix bugs, add issues, write examples, write software in LDPL, add statements to the [Standard Library](https://github.com/Lartu/ldpl-std), etc. Check out the [contribution guide](/CONTRIBUTING.md) for more information. Anything is very welcome! Even telling your friends about LDPL is a very easy and very useful contribution!

![Contributing Dinosaur](https://github.com/Lartu/ldpl/blob/master/images/tutorial-logo.png)

Contributors are expected to behave by the [LDPL Code of Conduct](/CODE_OF_CONDUCT.md). Tl;dr: be nice to everyone.

If you want to talk to the rest of us, you can find the LDPL community at [r/LDPL](https://reddit.com/r/LDPL) and via IRC on [irc.freenode.net](http://irc.freenode.net/), channel *\#ldpl*. Also on Telegram via the [ldpllang](https://t.me/ldpllang) group. The IRC and Telegram channels are bridged. You are welcome to open new LDPL channels anywhere else if you like.

### 🤠 Wanted List
This is a list of contributions we are actively looking for, so if you don't know where to start contributing, this might give you some inspiration.
 * **Language**
   * Expressions involving numbers and variables (maybe enclosed by something like `{}`).
     * Example: `store {foo + 1} in bar`
 * **Libraries**
   * A MySQL library.
   * A JSON library.
   * SQLite library.
   * Requests library.
 * **Others**
   * Updated man file from the latest docs.
   * Updated nano editor highlighting.
   * Add install instructions for extra files that might be needed to the installer.
 

## 🔎 Where can I get more help, if I need it?

You are welcome to submit an issue to this repository, check the [LDPL website](https://www.ldpl-lang.org) or join the community channels stated in the previous section. There are a few extra ways to get in touch with us stated on the website.

## 👔 LDPL Merchandise

There's also [LDPL merchandise available](https://www.teepublic.com/user/lartu), due to popular request! We've paired with TeePublic to bring you everything from shirts, to mousepads, to coffee mugs. All money made from selling these goodies goes to funding the language, paying for the server that keeps the LDPL website online, etc.

<img src="images/merch/shirt.jpg" width="150px">  <img src="images/merch/shirt2.jpg" width="150px"> <img src="images/merch/shirt3.jpg" width="150px">  <img src="images/merch/mug2.jpg" width="150px">

<img src="images/merch/mug3.jpg" width="150px"> <img src="images/merch/notebook.jpg" width="150px">  <img src="images/merch/sticker.jpg" width="150px">  <img src="images/merch/case.jpg" width="150px">

## 📜 License

The LDPL Compiler is distributed under the Apache 2.0 License. All LDPL Dinosaur logos where created by [Lartu](https://github.com/Lartu) and are released under a Creative Commons Attribution 4.0 International (CC BY 4.0) license.
